package com.caimei365.order.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.caimei365.order.components.ProductService;
import com.caimei365.order.mapper.BaseMapper;
import com.caimei365.order.mapper.OrderClubMapper;
import com.caimei365.order.model.ResponseJson;
import com.caimei365.order.model.enums.ReceivablesType;
import com.caimei365.order.model.po.BalanceRecordPo;
import com.caimei365.order.model.po.OrderPo;
import com.caimei365.order.model.po.OrderProductPo;
import com.caimei365.order.model.po.UserBeansHistoryPo;
import com.caimei365.order.model.vo.*;
import com.caimei365.order.service.RemoteCallService;
import com.caimei365.order.service.OrderClubService;
import com.caimei365.order.utils.ImageUtil;
import com.caimei365.order.utils.MathUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.google.common.util.concurrent.AtomicDouble;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.RandomStringUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/7/19
 */
@Slf4j
@Service
public class OrderClubServiceImpl implements OrderClubService {
    @Resource
    private BaseMapper baseMapper;
    @Resource
    private OrderClubMapper orderClubMapper;
    @Resource
    private ProductService productService;
    @Resource
    private RemoteCallService remoteCallService;
    @Value("${caimei.wwwDomain}")
    private String domain;

    /**
     * 机构订单列表
     *
     * @param userId     用户Id
     * @param orderState 1:待确认,2:待付款,3:待发货,4:已发货,5:退货款
     * @param orderNo    订单编号
     * @param beginTime  开始时间
     * @param endTime    截止时间
     * @param pageNum    页码
     * @param pageSize   每页数量
     */
    @Override
    public ResponseJson<PageInfo<OrderVo>> getOrderList(Integer userId, Integer orderState, String orderNo, String beginTime, String endTime, int pageNum, int pageSize) {
        // 分页请求数据
        PageHelper.startPage(pageNum, pageSize);
        List<OrderVo> orderList = orderClubMapper.getOrderList(userId, orderState, orderNo, beginTime, endTime);
        orderList.forEach(order -> {
            // 设置订单状态
            setOrderStatus(order);
            // 设置子订单数据
            getShopOrderData(order);
            // 设置付款金额
            getDiscernReceiptAndSetOrder(order);
        });
        PageInfo<OrderVo> pageInfo = new PageInfo(orderList);
        return ResponseJson.success(pageInfo);
    }

    /**
     * 根据关键词搜索订单
     *
     * @param userId     用户Id
     * @param searchWord 搜索关键词
     * @param pageNum    页码
     * @param pageSize   每页数量
     */
    @Override
    public ResponseJson<PageInfo<OrderVo>> getOrderListByKeyword(Integer userId, String searchWord, int pageNum, int pageSize) {
        // 设置搜索关键词历史记录
        setHistoryRecord(userId, searchWord);
        PageHelper.startPage(pageNum, pageSize);
        List<OrderVo> orderList = orderClubMapper.getOrderListByKeyword(searchWord, userId);
        //获取主订单数据
        orderList.forEach(order -> {
            // 设置订单状态
            setOrderStatus(order);
            // 设置子订单数据
            getShopOrderData(order);
            // 设置付款金额
            getDiscernReceiptAndSetOrder(order);
        });
        PageInfo<OrderVo> pageInfo = new PageInfo(orderList);
        return ResponseJson.success(pageInfo);
    }

    /**
     * 机构订单详情
     *
     * @param userId  用户Id
     * @param orderId 订单Id
     */
    @Override
    public ResponseJson<Map<String, Object>> getOrderDetail(Integer userId, Integer orderId) {
        OrderPo orderPo = orderClubMapper.getMainOrderByOrderId(orderId);
        if (null == orderPo) {
            return ResponseJson.error("该订单不存在！", null);
        }
        if (!userId.equals(orderPo.getUserId())) {
            return ResponseJson.error("无权限查看此订单！", null);
        }
        OrderVo order = new OrderVo();
        // OrderPo -> OrderVo
        BeanUtils.copyProperties(orderPo, order);
        order.setOrderMark("#" + order.getOrderId() + "#");
        // 设置订单状态
        setOrderStatus(order);
        // 设置子订单数据
        getShopOrderData(order);
        // 子订单列表
        List<ShopOrderVo> shopOrderList = order.getShopOrderList();
        AtomicDouble expensesOfTaxation = new AtomicDouble(0d);
        shopOrderList.forEach(shopOrder -> {
            List<OrderProductVo> orderProductList = shopOrder.getOrderProductList();
            orderProductList.forEach(orderProduct -> {
                // 是否充值商品
                boolean recharge = productService.isRechargeProduct(orderProduct.getProductId());
                if (recharge) {
                    order.setRechargeGoods(true);
                }
                // 总税费
                expensesOfTaxation.set(MathUtil.add(expensesOfTaxation.get(), orderProduct.getTotalAddedValueTax()).doubleValue());
                //已发货数量
                orderProduct.setShipmentsNum(orderProduct.getNum() + orderProduct.getPresentNum() - orderProduct.getNotOutStore());
                //已退货数量
                Integer returnedNum = orderClubMapper.countReturnedNum(orderProduct.getShopOrderId(), orderProduct.getProductId());
                returnedNum = null != returnedNum ? returnedNum : 0;
                orderProduct.setReturnedNum(returnedNum);
                //已取消发货数量
                Integer actualCancelNum = orderClubMapper.countActualCancelNum(orderProduct.getShopOrderId(), orderProduct.getProductId());
                actualCancelNum = null != actualCancelNum ? actualCancelNum : 0;
                orderProduct.setActualCancelNum(actualCancelNum);
                //判断商品价格是否含税
                boolean taxFlag = (1 == orderProduct.getIncludedTax() || (0 == orderProduct.getIncludedTax() && (1 == orderProduct.getInvoiceType() || 2 == orderProduct.getInvoiceType())));
                if (taxFlag) {
                    orderProduct.setIncludedTaxFlag(2);
                } else if (orderProduct.getIncludedTax() != null && 0 == orderProduct.getIncludedTax() && 3 == orderProduct.getInvoiceType()) {
                    orderProduct.setIncludedTaxFlag(1);
                } else {
                    orderProduct.setIncludedTaxFlag(3);
                }
            });
        });
        order.setExpensesOfTaxation(expensesOfTaxation.get());

        // 发票信息
        InvoiceVo invoice = baseMapper.getUserInvoice(userId);
        // 可用余额
        Double availableMoney = baseMapper.getAbleUserMoney(userId);
        // 付款金额
        List<DiscernReceiptVo> discernReceiptList = getDiscernReceiptAndSetOrder(order);
        // 退款记录
        List<ReturnedPurchaseVo> returnedPurchaseList = orderClubMapper.getReturnedPurchaseList(order.getOrderId());
        if (!returnedPurchaseList.isEmpty()) {
            AtomicDouble returnedPurchaseFee = new AtomicDouble(0d);
            returnedPurchaseList.forEach(returnedPurchase -> {
                returnedPurchaseFee.set(MathUtil.add(returnedPurchaseFee.get(), returnedPurchase.getRefundFee()).doubleValue());
            });
            //退款总金额
            order.setReturnedPurchaseFee(returnedPurchaseFee.get());
        }
        //售后条款
        ClauseVo clause = null;
        // 非二手
        if (0 == order.getSecondHandOrderFlag()) {
            Integer clauseId = order.getClauseId() == null ? 1 : order.getClauseId();
            clause = orderClubMapper.getClauseById(clauseId);
        }
        // 收货地址
        OrderUserinfoVo userInfo = orderClubMapper.getOrderUserinfo(orderId);

        Map<String, Object> map = new HashMap<>();
        map.put("order", order);
        map.put("shopOrderList", shopOrderList);
        map.put("orderInvoice", invoice);
        map.put("ableUserMoney", availableMoney);
        map.put("userInfo", userInfo);
        map.put("discernReceiptList", discernReceiptList);
        map.put("returnedPurchaseList", returnedPurchaseList);
        map.put("clause", clause);
        return ResponseJson.success(map);
    }

    /**
     * 设置订单状态
     */
    private void setOrderStatus(OrderVo order) {
        // 111, 待付待收待发
        if (11 == order.getStatus() && 1 == order.getPayStatus()) {
            order.setStatus(111);
        }
        // 判断交易全退情况下,是否发过货,77,交易全退可以查看物流
        int logisticsCount = orderClubMapper.countLogisticsBatch(order.getOrderId());
        if (7 == order.getStatus() && logisticsCount > 0) {
            order.setStatus(77);
        }
        // 判断二手订单情况下，若部分付款和已付款，排除退货/款的情况，且未确认打款供应商，10,添加确认打款供应商按钮
        if (1 == order.getSecondHandOrderFlag() && 0 == order.getRefundType() && 0 == order.getAffirmPaymentFlag()) {
            if (order.getStatus().toString().startsWith("2") || order.getStatus().toString().startsWith("3")) {
                // 之前是 “00”
                order.setAffirmPaymentFlag(10);
            }
        }
    }

    /**
     * 设置子订单数据
     */
    private void getShopOrderData(OrderVo order) {
        // 子订单
        String[] shopOrderIdArr = order.getShopOrderIds().split(",");
        List<String> shopOrderIds = Arrays.asList(shopOrderIdArr);
        List<ShopOrderVo> shopOrderList = orderClubMapper.getShopOrderList(shopOrderIds);
        shopOrderList.forEach(shopOrder -> {
            // 店铺促销活动
            PromotionsVo shopPromotion = null;
            if (null != shopOrder.getOrderPromotionsId() && shopOrder.getOrderPromotionsId() > 0) {
                shopPromotion = orderClubMapper.getOrderPromotionsById(shopOrder.getOrderPromotionsId());
                shopOrder.setShopPromotion(shopPromotion);
            }
            List<OrderProductVo> orderProductList = orderClubMapper.getShopOrderProduct(shopOrder.getShopOrderId());
            orderProductList.forEach(orderProduct -> {
                // 不含税可开票商品,单价/折后单价在原基础上加上税费
                boolean taxFlag = (0 == orderProduct.getIncludedTax() && (1 == orderProduct.getInvoiceType() || 2 == orderProduct.getInvoiceType()));
                if (taxFlag) {
                    orderProduct.setPrice(MathUtil.add(orderProduct.getPrice(), orderProduct.getAddedValueTax()).doubleValue());
                    orderProduct.setDiscountPrice(MathUtil.add(orderProduct.getPrice(), orderProduct.getAddedValueTax()).doubleValue());
                }
                orderProduct.setImage(ImageUtil.getImageUrl("product", orderProduct.getImage(), domain));
                // 查询订单下商品的促销活动
                if (null != orderProduct.getOrderPromotionsId() && orderProduct.getOrderPromotionsId() > 0) {
                    PromotionsVo promotions = orderClubMapper.getOrderPromotionsById(orderProduct.getOrderPromotionsId());
                    if (null != promotions) {
                        if (1 == promotions.getType() && 1 == promotions.getMode()) {
                            promotions.setTouchPrice(MathUtil.add(promotions.getTouchPrice(), MathUtil.div(MathUtil.mul(promotions.getTouchPrice(), orderProduct.getTaxRate()), 100)).doubleValue());
                        }
                        orderProduct.setProductPromotion(promotions);
                    }
                }
            });
            shopOrder.setOrderProductList(orderProductList);
            shopOrder.setShopLogo(ImageUtil.getImageUrl("shopLogo", shopOrder.getShopLogo(), domain));
        });
        // 过滤运费商品
        shopOrderList.removeIf(shopOrder -> shopOrder.getShopId() == 998);
        order.setShopOrderList(shopOrderList);
    }

    /**
     * 设置搜索关键词历史记录
     * @param userId 用户Id
     * @param searchWord 搜索关键词
     */
    private void setHistoryRecord(Integer userId, String searchWord) {
        // 初始化一个搜索关键词历史记录
        SearchHistoryVo historyRecord = new SearchHistoryVo();
        historyRecord.setUserId(userId);
        historyRecord.setSearchDate(new Date());
        historyRecord.setSearchWord(searchWord);
        historyRecord.setDelFlag(0);
        // 查询搜索关键词历史记录是否存在
        Integer recordId = orderClubMapper.getSearchHistoryIdByWord(searchWord);
        // 保存搜索关键词历史记录
        if (null != recordId && recordId >0) {
            historyRecord.setId(recordId);
            orderClubMapper.updateSearchHistory(historyRecord);
        } else {
            orderClubMapper.insertSearchHistory(historyRecord);
        }
        // 查询关键字历史记录条数
        int count = orderClubMapper.countSearchHistory(userId);
        // 只保留10条的搜索关键词历史记录
        if (count > 10) {
            orderClubMapper.deleteSearchHistoryLimit(userId);
        }
    }

    /**
     * 设置付款金额
     * @param order OrderVo
     */
    private List<DiscernReceiptVo> getDiscernReceiptAndSetOrder(OrderVo order) {
        // 支付记录
        List<DiscernReceiptVo> discernReceiptList = orderClubMapper.getDiscernReceipt(order.getOrderId());
        Double receiptAmount = 0d;
        // 订单款
        if (!discernReceiptList.isEmpty()) {
            AtomicDouble finalReceiptAmount = new AtomicDouble(0d);
            discernReceiptList.forEach(discernReceipt -> {
                finalReceiptAmount.set(MathUtil.add(finalReceiptAmount.get(), discernReceipt.getAssociateAmount()).doubleValue());
                if (null != discernReceipt.getPayType()) {
                    discernReceipt.setPayTypeStr(ReceivablesType.getReceivablesType(discernReceipt.getPayType()));
                }
            });
            receiptAmount = finalReceiptAmount.get();
        } else {
            //返佣款
            if (!order.getShopOrderIds().contains(",")) {
                Double tempAmount = orderClubMapper.getRebateAmountByShopOrder(Integer.parseInt(order.getShopOrderIds()));
                if (null == tempAmount) {
                    order.setOnlinePayFlag(0);
                } else {
                    order.setOnlinePayFlag(1);
                    receiptAmount = tempAmount;
                }
            }
        }
        // 判断是否可以走线上支付
        int offlineCount = orderClubMapper.countOfflinePayment(order.getOrderId());
        if (offlineCount > 0) {
            order.setOnlinePayFlag(1);
        } else {
            order.setOnlinePayFlag(0);
        }
        //付供应商总金额 + 默认手续费 > 订单总金额
        Double payTotalFee = order.getPayTotalFee();
        Double handlingFee = MathUtil.mul(payTotalFee, 0.0038, 2).doubleValue();
        if (MathUtil.compare(handlingFee, 8) < 0) {
            handlingFee = 8d;
        }
        List<Double> shouldPayShopAmount = orderClubMapper.getShouldPayShopAmountList(order.getOrderId());
        if (!shouldPayShopAmount.isEmpty()) {
            AtomicDouble finalHandlingFee = new AtomicDouble(handlingFee);
            shouldPayShopAmount.forEach(amount -> {
                finalHandlingFee.set(MathUtil.add(finalHandlingFee.get(), amount).doubleValue());
            });
            handlingFee = finalHandlingFee.get();
        }
        if (MathUtil.compare(payTotalFee, handlingFee) < 0) {
            order.setPayButton(true);
        } else {
            order.setPayButton(false);
        }
        //待付总金额
        order.setPendingPayments(MathUtil.sub(order.getPayableAmount(), receiptAmount).doubleValue());
        //支付总金额
        order.setReceiptAmount(receiptAmount);
        return discernReceiptList;
    }

    /**
     * 机构确认订单
     *
     * @param orderId 订单Id
     */
    @Override
    public ResponseJson<Void> confirmMainOrder(Integer orderId) {
        OrderPo order = orderClubMapper.getMainOrderByOrderId(orderId);
        if (null == order || 0 != order.getStatus()) {
            // 非待确认订单
            return ResponseJson.error("订单异常！", null);
        }
        Date date = new Date();
        String curDateStr = (new SimpleDateFormat("yyyy-MM-dd HH:mm:ss")).format(date);
        String note = "【确认订单】";
        List<OrderProductPo> orderProducts = orderClubMapper.getOrderProductList(orderId);
        orderProducts.forEach(product -> {
            // 是否二手商品
            boolean secondFlag = false;
            if (null != product.getProductCategory() && 2 == product.getProductCategory()) {
                secondFlag = true;
            }
            // 更新复购价格库
            productService.savePurchasePrice(order, product, secondFlag, note);
            // 二手商品确认商品如果是二手仪器则需要把该二手商品置为已售
            if (secondFlag) {
                orderClubMapper.updateSecondStatus(product.getProductId(), 1);
            }
        });
        if (null != order.getBalancePayFee() && MathUtil.compare(order.getBalancePayFee(), 0) > 0) {
            double userMoney = baseMapper.getUserMoney(order.getUserId());
            double updateMoney = MathUtil.sub(userMoney, order.getBalancePayFee()).doubleValue();
            // 扣除用户余额
            orderClubMapper.updateUserMoneyByUserId(updateMoney, order.getUserId());
            log.info(note + ">>>>>>>>>>>更新用户余额(update[user(userMoney)])userId：" + order.getUserId() + "，orderId：" + orderId);
            // 保存余额到收款记录
            productService.saveDiscernReceipt(order.getBalancePayFee(), orderId, note);
        }
        if (null == order.getBalancePayFee() || order.getBalancePayFee() == 0d) {
            order.setStatus(11);
            order.setReceiptStatus(1);
        } else if (null != order.getBalancePayFee() && MathUtil.compare(order.getBalancePayFee(), order.getPayTotalFee()) < 0) {
            order.setStatus(21);
            order.setReceiptStatus(2);
        } else {
            order.setStatus(31);
            order.setReceiptStatus(3);
        }
        order.setUpdateDate(curDateStr);
        order.setConfirmTime(curDateStr);
        //确认订单状态
        order.setConfirmFlag(2);
        // 更新订单状态
        orderClubMapper.updateOrderStatus(order);

        return ResponseJson.success("确认订单成功！", null);
    }

    /**
     * 机构取消订单
     *
     * @param orderId   订单Id
     * @param remark    备注（订单取消原因）
     */
    @Transactional
    @Override
    public ResponseJson<Void> cancelMainOrder(Integer orderId, String remark) {
        OrderPo order = orderClubMapper.getMainOrderByOrderId(orderId);
        boolean cancelFlag = (null != order && (0 == order.getStatus() || 11 == order.getStatus()));
        if (!cancelFlag) {
            // 非待确认(或待收待发)订单
            return ResponseJson.error("订单异常！", null);
        }
        Date date = new Date();
        String curDateStr = (new SimpleDateFormat("yyyy-MM-dd HH:mm:ss")).format(date);
        String note = "【取消订单】";
        // 余额退回原账户
        if (null != order.getBalancePayFee() && MathUtil.compare(order.getBalancePayFee(), 0) > 0) {
            double ableUserMoney = baseMapper.getAbleUserMoney(order.getUserId());
            double updateMoney = MathUtil.add(ableUserMoney, order.getBalancePayFee()).doubleValue();
            // 退回用户可用余额
            orderClubMapper.updateAbleUserMoneyByUserId(updateMoney, order.getUserId());
            log.info(note + ">>>>>>>>>>>可用余额退回原账户(update[user(ableUserMoney)])userId：" + order.getUserId() + "，orderId：" + orderId);
            // 取消收支记录
            BalanceRecordPo balanceRecord = new BalanceRecordPo();
            balanceRecord.setUserId(order.getUserId());
            balanceRecord.setOrderId(orderId);
            balanceRecord.setRemark("取消订单，余额记录无效");
            balanceRecord.setDelFlag(1);
            // 取消 余额收支记录
            orderClubMapper.cancelBalanceRecord(balanceRecord);
            log.info(note +">>>>>>>>>>>>取消余额收支记录(update[cm_user_balance_record])orderId：" + orderId);
        }
        order.setStatus(6);
        if (StringUtils.isBlank(remark)) {
            remark = "协销或用户主动取消订单";
        }
        order.setCloseReason(remark);
        order.setCloseTime(curDateStr);
        // 取消 订单
        orderClubMapper.cancelMainOrder(order);
        //二手订单取消订单, 二手商品标记未售
        if (1 == order.getSecondHandOrderFlag()) {
            List<OrderProductPo> orderProducts = orderClubMapper.getOrderProductList(orderId);
            orderProducts.forEach(product -> {
                // 是否二手商品
                boolean secondFlag = false;
                if (null != product.getProductCategory() && 2 == product.getProductCategory()) {
                    secondFlag = true;
                }
                // 二手仪器标记未售
                if (secondFlag) {
                    orderClubMapper.updateSecondStatus(product.getProductId(), 0);
                }
            });
        }
        // 退回 采美豆
        if (null != order.getUserBeans() && order.getUserBeans() > 0) {
            UserBeansHistoryPo beansHistory = new UserBeansHistoryPo();
            beansHistory.setUserId(order.getUserId());
            beansHistory.setOrderId(orderId);
            beansHistory.setBeansType(9);
            beansHistory.setType(1);
            beansHistory.setNum(order.getUserBeans());
            beansHistory.setPushStatus(0);
            beansHistory.setAddTime(date);
            beansHistory.setDelFlag(0);
            // 用户采美豆
            Integer userBeans = baseMapper.getUserBeans(order.getUserId());
            // 更新用户剩余采美豆数量
            int beans = userBeans + beansHistory.getNum();
            productService.updateUserBeans(beansHistory, beans, note);
        }
        //取消订单短信推送
        String bindMobile = baseMapper.getBindMobileByUserId(order.getUserId());
        if (StringUtils.isNotBlank(bindMobile)) {
            String shortLink = remoteCallService.getShortLink(8, 11, domain + "/user/order/detail.html?orderId=" + orderId);
            String content = "您已成功取消订单（订单编号：" + order.getOrderNo() + "），订单金额￥" + order.getPayTotalFee() + "。您可关注采美公众号或者访问采美微信小程序和网站查看订单。" +
                    "平台公众号：微信搜索“采美365网”; 微信小程序：微信搜索“采美采购商城”；网址：www.caimei365.com/t/" + shortLink;
            boolean sendSms = remoteCallService.getSendSms(11, bindMobile, content);
            if (!sendSms) {
                log.info("取消订单推送失败，orderId>>>>" + orderId);
            }
        }

        return ResponseJson.success("取消订单成功！", null);
    }

    /**
     * 机构删除订单
     *
     * @param orderId 订单Id
     */
    @Override
    public ResponseJson<Void> deleteMainOrder(Integer orderId) {
        OrderPo order = orderClubMapper.getMainOrderByOrderId(orderId);
        if (null == order || 6 != order.getStatus()) {
            // 非已关闭订单
            return ResponseJson.error("订单异常！", null);
        }
        // 逻辑删除主订单
        order.setDelFlag(1);
        order.setNote("用户发起删除订单");
        orderClubMapper.deleteMainOrder(order);
        log.info("【删除订单】>>>>>>>>>逻辑删除主订单(update[cm_order])orderId：" + orderId);
        // 逻辑删除子订单
        String[] shopOrderIdArr = order.getShopOrderIds().split(",");
        List<String> shopOrderIds = Arrays.asList(shopOrderIdArr);
        orderClubMapper.deleteShopOrder(shopOrderIds);
        log.info("【删除订单】>>>>>>>>>逻辑删除子订单(update[cm_shop_order])shopOrderIds：" + order.getShopOrderIds());
        // 逻辑删除收款记录
        List<String> receiptIds = orderClubMapper.getOrderReceiptIds(orderId);
        if (!receiptIds.isEmpty()) {
            orderClubMapper.deleteOrderReceiptRelation(orderId);
            orderClubMapper.deleteDiscernReceipt(receiptIds);
            log.info("【删除订单】>>>>>>>>>逻辑删除收款记录(update[cm_discern_receipt])orderId：" + orderId);
        }
        // 逻辑删除付款记录
        List<String> payIds = orderClubMapper.getPayShopIds(shopOrderIds);
        if (!payIds.isEmpty()) {
            orderClubMapper.deletePayShopRecord(shopOrderIds);
            orderClubMapper.deletePayShop(payIds);
            log.info("【删除订单】>>>>>>>>>逻辑删除付款记录(update[cm_pay_shop])shopOrderIds：" + order.getShopOrderIds());
        }

        return ResponseJson.success("删除订单成功！", null);
    }

    /**
     * 机构确认收货
     *
     * @param orderId 订单Id
     */
    @Override
    public ResponseJson<Void> receiveGoods(Integer orderId) {
        OrderPo order = orderClubMapper.getMainOrderByOrderId(orderId);
        if (null == order) {
            return ResponseJson.error("订单不存在！", null);
        }
        if (33 == order.getStatus() && 3 == order.getPayStatus()) {
            // 已付款供应商,订单完成
            order.setStatus(5);
        } else if (33 == order.getStatus()) {
            // 交易完成
            order.setStatus(4);
        } else {
            return ResponseJson.error("订单异常！", null);
        }
        Date date = new Date();
        String curDateStr = (new SimpleDateFormat("yyyy-MM-dd HH:mm:ss")).format(date);
        String note = "【确认收货】";
        order.setUpdateDate(curDateStr);
        // 更新订单状态
        orderClubMapper.updateOrderStatus(order);
        // 更新物流状态
        orderClubMapper.updateLogisticsBatch(orderId);

        // 主动确认收货，赠送100采美豆
        UserBeansHistoryPo beansHistory = new UserBeansHistoryPo();
        beansHistory.setUserId(order.getUserId());
        beansHistory.setOrderId(orderId);
        beansHistory.setBeansType(7);
        beansHistory.setType(1);
        beansHistory.setNum(100);
        beansHistory.setPushStatus(0);
        beansHistory.setAddTime(date);
        beansHistory.setDelFlag(0);
        // 用户采美豆
        Integer userBeans = baseMapper.getUserBeans(order.getUserId());
        // 更新用户剩余采美豆数量
        int beans = userBeans + beansHistory.getNum();
        productService.updateUserBeans(beansHistory, beans, note);

        return ResponseJson.success("确认收货成功！", null);
    }

    /**
     * 根据用户Id查找订单搜索历史记录
     *
     * @param userId 用户Id
     */
    @Override
    public ResponseJson<List<SearchHistoryVo>> getOrderSearchHistory(Integer userId) {
        List<SearchHistoryVo> historyList = orderClubMapper.getOrderSearchHistory(userId);
        return ResponseJson.success(historyList);
    }

    /**
     * 根据用户Id删除订单搜索历史记录
     *
     * @param userId 用户Id
     */
    @Override
    public ResponseJson<Void> deleteOrderSearchHistory(Integer userId) {
        orderClubMapper.deleteOrderSearchHistory(userId);
        return ResponseJson.success(null);
    }

    /**
     * 物流详情
     *
     * @param orderId 订单Id
     */
    @Override
    public ResponseJson<List<LogisticsBatchVo>> getOrderLogistics(Integer orderId) {
        int orderCount = orderClubMapper.countMainOrderByOrderId(orderId);
        if (0 == orderCount) {
            return ResponseJson.error("订单不存在！", null);
        }
        // 获取物流批次列表
        List<LogisticsBatchVo> batchList = orderClubMapper.getLogisticsBatchList(orderId);
        batchList.forEach(batch -> {
            // 根据物流批次获取子订单
            List<String> shopOrderIds = orderClubMapper.getBatchShopOrderIds(orderId, batch.getId());
            List<ShopOrderVo> shopOrderList = orderClubMapper.getShopOrderList(shopOrderIds);
            shopOrderList.forEach(shopOrder -> {
                shopOrder.setShopLogo(ImageUtil.getImageUrl("shopLogo", shopOrder.getShopLogo(), domain));
                // 获取发货物流记录
                List<LogisticsRecordVo> logisticsRecordList = orderClubMapper.getLogisticsRecord(shopOrder.getShopOrderId(), batch.getId());
                shopOrder.setLogisticsRecordList(logisticsRecordList);
            });
            // 获取物流信息
            List<LogisticsInformationVo> logisticsInfoList = orderClubMapper.getLogisticsInfoList(batch.getId());
            logisticsInfoList.forEach(logisticsInfo -> {
                List<LogisticsRouterVo> routers = JSONArray.parseArray(logisticsInfo.getInfo(), LogisticsRouterVo.class);
                if (!CollectionUtils.isEmpty(routers)) {
                    routers.forEach(router -> {
                        if (StringUtils.isEmpty(router.getDesc())) {
                            router.setDesc(router.getContext());
                        }
                    });
                    logisticsInfo.setRouterList(routers);
                }
            });
            batch.setLogisticsInformationList(logisticsInfoList);
            batch.setShopOrderList(shopOrderList);
        });

        return ResponseJson.success(batchList);
    }

    /**
     * 分享订单，获取分享码
     *
     * @param orderId 订单Id
     */
    @Override
    public ResponseJson<String> getOrderShareCode(Integer orderId) {
        int orderCount = orderClubMapper.countMainOrderByOrderId(orderId);
        if (0 == orderCount) {
            return ResponseJson.error("订单不存在！", null);
        }
        // 生成分享码
        String shareCode = RandomStringUtils.random(4, false, true);
        // 过期时间
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        calendar.add(Calendar.DATE, 1);
        // 获取数据库已有分享码
        OrderShareCodeVo orderCodeVo = orderClubMapper.getOrderCodeById(orderId);
        if (orderCodeVo != null && orderCodeVo.getExpiredTime().compareTo(new Date()) >= 0 ) {
            // 旧分享码还未过期，直接返回
            return ResponseJson.success(orderCodeVo.getShareCode());
        } else if (orderCodeVo != null) {
            // 如果旧分享码已经过期，删除分享码使用记录
            orderClubMapper.deleteOrderShareCodeRecord(orderId);
            // 更新数据库的分享码
            orderCodeVo.setShareCode(shareCode);
            orderCodeVo.setExpiredTime(calendar.getTime());
            orderCodeVo.setDelFlag(0);
            orderClubMapper.updateOrderShareCode(orderCodeVo);
        } else {
            // 如果没有分享记录,新增数据库的分享码
            OrderShareCodeVo newShareCodeVo = new OrderShareCodeVo();
            newShareCodeVo.setOrderId(orderId);
            newShareCodeVo.setShareCode(shareCode);
            newShareCodeVo.setExpiredTime(calendar.getTime());
            newShareCodeVo.setAddTime(new Date());
            newShareCodeVo.setDelFlag(0);
            orderClubMapper.insertOrderShareCode(newShareCodeVo);
        }
        // 返回新生成的分享码
        return ResponseJson.success(shareCode);
    }

    /**
     * 分享订单，校验分享码
     *
     * @param orderId           订单Id
     * @param userId            用户Id
     * @param serviceProviderId 协销Id
     * @param shareCode         分享码
     * @param code              微信code
     * @param encryptedData     微信加密数据
     * @param iv                微信加密算法的初始向量
     * @param headers           HttpHeaders
     */
    @Override
    public ResponseJson<String> checkOrderShareCode(Integer orderId, Integer userId, Integer serviceProviderId, String shareCode, String code, String encryptedData, String iv, HttpHeaders headers) {
        int orderCount = orderClubMapper.countMainOrderByOrderId(orderId);
        if (0 == orderCount) {
            return ResponseJson.error("订单不存在！", null);
        }
        // 微信小程序授权登录（调用user服务，发送服务间调用POST请求）
        String loginData = remoteCallService.appletsAuthorization(code, encryptedData, iv, headers);
        if (StringUtils.isEmpty(loginData)){
            return ResponseJson.error("微信授权登录异常！", null);
        }
        // 解析登录后得到 userIdentity,  openId等
        JSONObject userInfo = JSONObject.parseObject(loginData);
        String openId = userInfo.getString("openId");
        if (StringUtils.isEmpty(openId)) {
            return ResponseJson.error("微信授权登录异常！", null);
        }
        // 用户身份: 1协销 2会员机构 3供应商 4普通机构
        Integer userIdentity = userInfo.getInteger("userIdentity");
        Integer loginUserId = userInfo.getInteger("userId");
        Integer loginSpId = userInfo.getInteger("serviceProviderId");
        // 协销身份
        boolean spFlag = (null != userIdentity && 1 == userIdentity);
        // 机构身份
        boolean clubFlag = (null != userIdentity && (2 == userIdentity || 4 ==userIdentity));
        if (spFlag) {
            if (null != serviceProviderId && serviceProviderId.equals(loginSpId)) {
                // 该机构的协销
                return ResponseJson.success(2, "该机构的协销查看分享订单!", loginData);
            } else {
                // 非该机构协销（游客）
                return ResponseJson.success(0,"非该机构协销查看分享订单！", null);
            }
        } else if (clubFlag && userId.equals(loginUserId)) {
            // 该机构的运营人员, 旧接口返回  0,true
            return ResponseJson.success(1, "该机构的运营人员查看分享订单!", loginData);
        }
        /*
         * 其他按游客权限处理
         */
        // 根据openId统计数据库内是否有有效分享码
        int codeCount = orderClubMapper.countShareCodeRecord(orderId, openId);
        if (codeCount > 0) {
            // 游客权限
            return ResponseJson.success(0,"游客权限查看分享订单！", null);
        }
        // 根据订单Id获取分享码
        OrderShareCodeVo orderCodeVo = orderClubMapper.getOrderCodeById(orderId);
        if (null == orderCodeVo) {
            return ResponseJson.error("该订单没有生成分享码！", null);
        }
        // 校验分享码
        if (orderCodeVo.getShareCode().equals(shareCode)){
            if (orderCodeVo.getExpiredTime().compareTo(new Date()) > 0) {
                // 分享码有效, 保存分享码使用记录
                OrderShareCodeRecordVo codeRecord = new OrderShareCodeRecordVo();
                codeRecord.setOpenId(openId);
                codeRecord.setAddTime(new Date());
                codeRecord.setOrderId(orderCodeVo.getOrderId());
                codeRecord.setDelFlag(0);
                codeRecord.setShareCodeId(orderCodeVo.getId());
                orderClubMapper.insertOrderShareCodeRecord(codeRecord);
                // 返回
                return ResponseJson.success(0,"游客权限查看分享订单！", null);
            } else {
                // 分享码过期
                return ResponseJson.error("分享码已失效！", null);
            }
        } else {
            return ResponseJson.error("分享码不正确！", null);
        }
    }

    /**
     * 分享订单，初始商品数据
     *
     * @param orderId 订单Id
     */
    @Override
    public ResponseJson<List<OrderProductVo>> getOrderShareInitProduct(Integer orderId) {
        List<OrderProductVo> orderProductList = orderClubMapper.getOrderProductByOrderId(orderId);
        orderProductList.forEach(orderProduct -> orderProduct.setImage(ImageUtil.getImageUrl("product", orderProduct.getImage(), domain)));
        return ResponseJson.success(orderProductList);
    }

    /**
     * 二手订单 确认打款供应商
     *
     * @param orderId 订单Id
     * @param userId  用户Id
     */
    @Override
    public ResponseJson<Void> secondPaymentConfirm(Integer orderId, Integer userId) {
        OrderPo orderPo = orderClubMapper.getMainOrderByOrderId(orderId);
        if (null == orderPo) {
            return ResponseJson.error("该订单不存在！", null);
        }
        if (!userId.equals(orderPo.getUserId())) {
            return ResponseJson.error("需机构自身确认打款供应商！", null);
        }
        boolean flag = 1 == orderPo.getSecondHandOrderFlag() && 0 == orderPo.getRefundType() && 0 == orderPo.getAffirmPaymentFlag();
        if (flag) {
            if (orderPo.getStatus().toString().startsWith("3") && 3 == orderPo.getPayStatus()) {
                orderPo.setAffirmPaymentFlag(1);
                orderPo.setStatus(5);
            } else if (orderPo.getStatus().toString().startsWith("3")) {
                orderPo.setAffirmPaymentFlag(1);
                orderPo.setStatus(4);
            } else if (orderPo.getStatus().toString().startsWith("2")) {
                orderPo.setAffirmPaymentFlag(1);
            } else {
                return ResponseJson.error("订单状态异常！", null);
            }
            // 更新订单状态
            orderClubMapper.updateOrderStatus(orderPo);
        } else {
            return ResponseJson.error("订单状态异常！", null);
        }
        return ResponseJson.success(null);
    }

}
